<?php
/**
 * Cache servvice factory
 */

namespace Clickio\CacheControl;

use Clickio\Logger as log;
use Clickio\Utils\SafeAccess;

/**
 * Cache service factory
 *
 * @package CacheControll
 */
class CacheServiceFactory
{

    const ACTIONS_NS = "Services";
    const SERVICES_DIR = CLICKIO_PLUGIN_DIR."/src/CacheControl/Services";
    /**
     * Create cleaner service
     *
     * @param string $name service name
     *
     * @return ICacheService
     */
    public static function create(string $name): Interfaces\ICacheService
    {
        $ns_list = explode('\\', $name);
        if (count($ns_list) == 1) {
            $name = static::_convertToFQCN($name);
        }
        $host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        $logger = log\Logger::getLogger($host);

        if (class_exists($name)) {
            $cls = new $name($logger);
            return $cls;
        }

        throw new \Exception("Undefined cleaner $name");
    }

    /**
     * Convert natural name into "Fully qualified class name"
     * By historical reasons class names for cache services
     * uses natural names without namespace
     *
     * @param string $cls class name without namespace
     *
     * @return string
     */
    private static function _convertToFQCN(string $cls): string
    {
        return  sprintf("%s\%s\%s", __NAMESPACE__, static::ACTIONS_NS, $cls);
    }

    /**
     * Instantiate all services
     *
     * @return array
     */
    public static function createAll(): array
    {
        $services = [];
        foreach (scandir(static::SERVICES_DIR) as $file) {
            if (in_array($file, ['.', '..']) || empty($file)) {
                continue;
            }
            $name= basename($file, '.php');
            try{
                $services[] = static::create($name);
            } catch (\Exception $err){
                continue ;
            }
        }
        return $services;
    }
}