<?php
/**
 * Abstract extra content service
 */
namespace Clickio\ExtraContent\Services;

use Clickio\Logger\Interfaces\ILogger;

/**
 * Abstract content service
 *
 * @package ExtraContent\Services
 */
abstract class ContentServiceBase
{

    /**
     * Logger instance
     *
     * @var ILogger
     */
    private $_log = null;

    /**
     * Constructor
     *
     * @param ILogger $logger Logger instance
     */
    public function __construct(ILogger $logger)
    {
        $this->_log = $logger;
    }

    /**
     * Logger method wrapper.
     * See Logger docs for more info
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function info(string $msg)
    {
        $msg = $this->_formatLogPrefix($msg);
        $this->_log->info($msg);
    }

    /**
     * Logger method wrapper.
     * See Logger docs for more info
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function error(string $msg)
    {
        $msg = $this->_formatLogPrefix($msg);
        $this->_log->error($msg);
    }

    /**
     * Logger method wrapper.
     * See Logger docs for more info
     *
     * @param string $msg log message
     * @param array $debug debug info
     *
     * @return void
     */
    protected function debug(string $msg, array $debug = [])
    {
        $msg = $this->_formatLogPrefix($msg);
        $this->_log->debug($msg, $debug);
    }

    /**
     * Logger message formating
     *
     * @param string $msg source message
     *
     * @return string
     */
    private function _formatLogPrefix(string $msg): string
    {
        return sprintf("%s: %s", "Extra content service ".static::class, $msg);
    }
}