<?php

/**
 * Abstract Cache service
 */

namespace Clickio\Prism\Cache\Engine;

use Clickio\Logger\Interfaces\ILogger;

/**
 * Base cache service
 *
 * @package Prism\Cache\Engine
 */
abstract class AbstractCacheService
{

    /**
     * Logger instance
     *
     * @var ILogger
     */
    private $_log = null;

    /**
     * Constructor
     *
     * @param ILogger $logger logger instance
     */
    public function __construct(ILogger $logger)
    {
        $this->_log = $logger;
    }

    /**
     * Log notice message
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function notice(string $msg)
    {
        $msg = sprintf('%s: %s', static::class, $msg);
        $this->_log->notice($msg);
    }

    /**
     * Log info message
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function info(string $msg)
    {
        $msg = sprintf('%s: %s', static::class, $msg);
        $this->_log->info($msg);
    }

    /**
     * Log warning message
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function warning(string $msg)
    {
        $msg = sprintf('%s: %s', static::class, $msg);
        $this->_log->warning($msg);
    }

    /**
     * Log error message
     *
     * @param string $msg log message
     *
     * @return void
     */
    protected function error(string $msg)
    {
        $msg = sprintf('%s: %s', static::class, $msg);
        $this->_log->error($msg);
    }

    /**
     * Log debug message
     *
     * @param string $msg log message
     * @param array $dbg debug info
     *
     * @return void
     */
    protected function debug(string $msg, array $dbg = [])
    {
        $msg = sprintf('%s: %s', static::class, $msg);
        $this->_log->debug($msg, $dbg);
    }

    /**
     * Get cached data
     *
     * @param string $key cache key
     *
     * @return string
     */
    public function get(string $key): string
    {
        return '';
    }

    /**
     * Put some data into cache
     *
     * @param string $key cache key
     * @param mixed $value cache value
     *
     * @return bool
     */
    public function add(string $key, $value): bool
    {
        return true;
    }

    /**
     * Purge cache for a key
     *
     * @param string $key cache key
     *
     * @return bool
     */
    public function purge(string $key): bool
    {
        return true;
    }

    /**
     * Purge all
     *
     * @return bool
     */
    public function purgeAll(): bool
    {
        return true;
    }
}