<?php
/**
 * Rest api action
 */

namespace Clickio\RestApi\Actions;

use Clickio\Integration\IntegrationServiceFactory;
use Clickio\Listings\FilterFacade;
use Clickio\RestApi as rest;
use Clickio\Utils\SafeAccess;

/**
 * List posts by params
 *
 * Example:
 *      POST http://domain.name/wp-json/clickio/lists/
 *      [{
 *          id: 1,
 *          order: DESC,
 *          orderby: some_field
 *      }]
 *
 * @package RestApi\Actions
 */
class GetLists extends rest\BaseRestAction implements rest\Interfaces\IRestApi
{
    /**
     * Handle http post method
     *
     * @return mixed
     */
    public function post()
    {
        $body = $this->request->get_body();
        $params = json_decode($body, true);
        if (empty($params)) {
            $params = [[]];
        }

        $first_query = SafeAccess::fromArray($params, 0, 'array', false);
        if (!is_array($first_query)) {
            return [];
        }

        $ordering_plugin = IntegrationServiceFactory::getService('taxonomy_order');
        $ordering_plugin::disable();

        ob_start();
        $filter = FilterFacade::create();
        $items = $filter->filter($params);
        ob_clean();
        ob_get_clean();
        ob_end_clean();
        return $items;
    }
}