<?php
/**
 * Rest api action
 */

namespace Clickio\RestApi\Actions;

use Clickio\RestApi as rest;

/**
 * Get items by id
 *
 * Example:
 *      GET http://domain.name/wp-json/clickio/menus/(?P<id>\d+)/
 *
 * @package RestApi\Actions
 */
class GetMenu extends rest\BaseRestAction implements rest\Interfaces\IRestApi
{
    /**
     * Handle http get method
     *
     * @return mixed
     */
    public function get()
    {
        $id = $this->request->get_param('id');
        $args = array(
                'order'                  => 'ASC',
                'orderby'                => 'menu_order',
                'output'                 => "ARRAY_A",
                'output_key'             => 'menu_order',
                'update_post_term_cache' => false,
            );
        $items = [];
        $res=[];
        foreach (wp_get_nav_menu_items($id, $args) as $item) {
            if ($item->menu_item_parent == 0) {
                $res[$item->ID]=[
                    'WpId' => $item->ID,
                    'DisableSwipe' => 0,
                    'Name'  => $item->title,
                    'Order' => $item->menu_order,
                    'Url'   => $item->url,
                ];
                $items[$item->ID] = &$res[$item->ID];
            } else {
                $items[$item->menu_item_parent]['Children'][] = [
                    'WpId' => $item->ID,
                    'DisableSwipe' => 0,
                    'Name'  => $item->title,
                    'Order' => $item->menu_order,
                    'Url'   => $item->url,
                    'parent' => $item->menu_item_parent,
                ];
                $items[$item->ID] = &$items[$item->menu_item_parent]
                                           ['Children']
                                           [count($items[$item->menu_item_parent]['Children'])-1];
            }
        }
        return ["MenuItems"=> array_values($res)];
    }
}